@@ -26,7 +26,7 @@ LOGGER = logging.getLogger("bakalariapi")
 LOGGER.addHandler(logging.NullHandler())
 
 __all__ = ["Endpoint", "BakalariAPI", "LAST_SUPPORTED_VERSION", "GetMode"]
-LAST_SUPPORTED_VERSION = "1.43.824.1"
+LAST_SUPPORTED_VERSION = "1.45"
 
 
 class Endpoint:
@@ -39,6 +39,7 @@ class Endpoint:
     KOMENS_GET = "/next/komens.aspx/GetMessageData"
     KOMENS_CONFIRM = "/next/komens.aspx/SetMessageConfirmed"
     FILE = "/next/getFile.aspx"
+    ROZVRH = "/next/rozvrh.aspx"
     GRADES = "/next/prubzna.aspx"
     SESSION_INFO = "/sessioninfo"
     SESSION_EXTEND = "/sessionextend"
@@ -187,8 +188,6 @@ def _resolve(
 def is_version_supported(version: str):
     """Zkontroluje, jestli `BakaláriAPI` podporuje danou verzi Bakalářů.
 
-    V současné chvíli pouze ověřuje, zda verze Bakalářů je shodná s `LAST_SUPPORTED_VERSION`.
-
     Args:
         version:
             Verze, která se má zkontrolovat.
@@ -196,7 +195,7 @@ def is_version_supported(version: str):
     Returns:
         Vrátí `True` pokud se shodují, jinak `False`.
     """
-    return version == LAST_SUPPORTED_VERSION
+    return version.startswith(LAST_SUPPORTED_VERSION)
 
 
 class GetMode(Enum):
@@ -361,8 +360,6 @@ class BakalariAPI:
     def is_version_supported(self):
         """Zkontroluje, jestli `BakaláriAPI` podporuje verzi Bakalářů, která je na serveru.
 
-        V současné chvíli pouze ověřuje, zda verze Bakalářů je shodná s `LAST_SUPPORTED_VERSION`.
-
         Returns:
             Vrátí `True` pokud se shodují, jinak `False`.
             Pokud verze Bakalářů nebyla získána (tzn. je `None`), vrátí `False`.
