@@ -1,5 +1,8 @@
 """Prakticky pouze placeholder, aby šlo spustit `bakalarishell` přímo ze zdroje ze složky nebo přes `python -m bakalarishell`"""
 
-from main import main
+try:
+    from .main import main
+except ImportError:
+    from main import main  # type: ignore
 
 main()
